@@ -4,12 +4,14 @@ import socket
 import json
 import asyncio
 import uuid
-from typing import Dict, Tuple, Union
+from typing import Dict, Tuple, Union, Optional
 
 import zmq
 import zmq.asyncio
 from zmq.sugar.socket import Socket
 
+from fastapi import WebSocket
+
 
 channel_socket_types = {
     "hb": zmq.REQ,
@@ -85,19 +87,40 @@ async def launch_kernel(
     return p
 
 
-def create_socket(channel: str, cfg: cfg_t) -> Socket:
+def create_socket(channel: str, cfg: cfg_t, identity: Optional[bytes] = None) -> Socket:
     ip = cfg["ip"]
     port = cfg[f"{channel}_port"]
     url = f"tcp://{ip}:{port}"
     socket_type = channel_socket_types[channel]
     sock = context.socket(socket_type)
     sock.linger = 1000  # set linger to 1s to prevent hangs at exit
+    if identity:
+        sock.identity = identity
     sock.connect(url)
     return sock
 
 
-def connect_channel(channel_name: str, cfg: cfg_t) -> Socket:
-    sock = create_socket(channel_name, cfg)
+def connect_channel(
+    channel_name: str, cfg: cfg_t, identity: Optional[bytes] = None
+) -> Socket:
+    sock = create_socket(channel_name, cfg, identity)
     if channel_name == "iopub":
         sock.setsockopt(zmq.SUBSCRIBE, b"")
     return sock
+
+
+class AcceptedWebSocket:
+    _websocket: WebSocket
+    _accepted_subprotocol: Optional[str]
+
+    def __init__(self, websocket, accepted_subprotocol):
+        self._websocket = websocket
+        self._accepted_subprotocol = accepted_subprotocol
+
+    @property
+    def websocket(self):
+        return self._websocket
+
+    @property
+    def accepted_subprotocol(self):
+        return self._accepted_subprotocol
